# Code by Burns Hudson Copyright (C) 2013
#
#    main.py
# Python script that runs on the MK802ii that handles image processing
# and arduino communication for motion tracking laser targeting
#
# for calibration, move the laser to the bottom right of the webcam
# using wasd/tfgh, press z, then move the laser to the top left using
# wasd/tfgh, and then press space bar to begin tracking
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import serial
import thread
from random import randint
import time


#Set port for arduino communication
s = serial.Serial(port='/dev/ttyACM0', baudrate=9600)

#use new cv library in place of old
import cv2.cv as cv

class Target:

    def allowSend(self):
        global theText
		
        #looping to read from arduino
        while True:
            buffer = ''
            buffer += s.read(3)
            print(buffer)
            if 'dun' in buffer:
                self.allowed=True
            #Check to see if buffer was caught part-way in and reset
            if 'ndu' in buffer:
                buffer = ''
            if 'und' in buffer:
                buffer = ''
                
    def __init__(self):
        self.capture = cv.CaptureFromCAM(0)

        cv.NamedWindow("Target", 1)
        thread.start_new_thread(self.allowSend, ())

        time.sleep(5)
        self.allowed = False
        #Calibration for laser movement
        self.xVal = -480
        self.yVal = -270
        self.degX = 69.0
        self.degY = 45.0
        self.processing = True
		
    def run(self):
        # Capture first frame to get size
        frame = cv.QueryFrame(self.capture)
        frame_size = cv.GetSize(frame)
        color_image = cv.CreateImage(cv.GetSize(frame), 8, 3)
        grey_image = cv.CreateImage(cv.GetSize(frame), cv.IPL_DEPTH_8U, 1)
        moving_average = cv.CreateImage(cv.GetSize(frame), cv.IPL_DEPTH_32F, 3)

        first = True
        
        while True:
            
            closest_to_left = cv.GetSize(frame)[0]
            closest_to_right = cv.GetSize(frame)[1]
            color_image = cv.QueryFrame(self.capture)

            if self.processing:
        
                # Smooth to get rid of false positives
                cv.Smooth(color_image, color_image, cv.CV_GAUSSIAN, 3, 0)

                if first:
                    difference = cv.CloneImage(color_image)
                    temp = cv.CloneImage(color_image)
                    cv.ConvertScale(color_image, moving_average, 1.0, 0.0)
                    first = False
                else:
                    cv.RunningAvg(color_image, moving_average, 0.020, None)

                # Convert the scale of the moving average.
                cv.ConvertScale(moving_average, temp, 1.0, 0.0)

                # Minus the current frame from the moving average.
                cv.AbsDiff(color_image, temp, difference)

                # Convert the image to grayscale.
                cv.CvtColor(difference, grey_image, cv.CV_RGB2GRAY)

                # Convert the image to black and white.
                cv.Threshold(grey_image, grey_image, 70, 255, cv.CV_THRESH_BINARY)

                # Dilate and erode to get people blobs
                cv.Dilate(grey_image, grey_image, None, 18)
                cv.Erode(grey_image, grey_image, None, 10)

                storage = cv.CreateMemStorage(0)
                contour = cv.FindContours(grey_image, storage, cv.CV_RETR_CCOMP, cv.CV_CHAIN_APPROX_SIMPLE)
                points = []
                
                while contour:
                    bound_rect = cv.BoundingRect(list(contour))
                    contour = contour.h_next()

                    pt1 = (bound_rect[0], bound_rect[1])
                    pt2 = (bound_rect[0] + bound_rect[2], bound_rect[1] + bound_rect[3])
                    points.append(pt1)
                    points.append(pt2)
                    cv.Rectangle(color_image, pt1, pt2, cv.CV_RGB(255,0,0), 1)

                if len(points):	
                    center_point = reduce(lambda a, b: ((a[0] + b[0]) / 2, (a[1] + b[1]) / 2), points)
                    cv.Circle(color_image, center_point, 40, cv.CV_RGB(255, 255, 255), 1)
                    cv.Circle(color_image, center_point, 30, cv.CV_RGB(255, 100, 0), 1)
                    cv.Circle(color_image, center_point, 20, cv.CV_RGB(255, 255, 255), 1)
                    cv.Circle(color_image, center_point, 10, cv.CV_RGB(255, 100, 0), 1)

                    #Figure out where the real-world coordinates are
                    ratiox = 1.0/(320.0/self.degX)
                    ratioy = 1.0/(240.0/self.degY)
                    
                    #Figure out where the real-world coordinates are (cont)
                    xPoint = self.degX-center_point[0]*ratiox
                    yPoint = self.degY-center_point[1]*ratioy
                    theString = ""
                    theString +=(str(xPoint))
                    theString += ","
                    theString +=(str(yPoint))
                    
                    #Random speed to simulate non-mechanical laser movement
                    theSpeed = (str(randint(2000,10000))) #16383 is max stable speed at 12v
                    theSpeed += "*"

                    #Is movement currently allowed (AKA not calibrating)
                    if(self.allowed == True):
                        print(theString)
                        s.write(str(theSpeed))
                        s.write(str(theString))
                        
                        self.allowed = False

            #hidden to speed processing. Manually unhide when calibrating
            #cv.ShowImage("Target", color_image)
            

            # Listen for ESC key
            c = cv.WaitKey(7) % 0x100

            # Listen for all the other special keys.
            # wasd for precise movement
            # tfgh for 10x movement
            # i for laser on/off
            # o for reset to 0,0 (should be done every time before powering down to prevent need for recalibration)
            # z for 'set current position as 0,0'
            # r for arduino reset
            if c==32:
                s.write("0.0,0.0")
                self.processing = True
                #self.degX = 360.0/(4096.0/(0-self.xVal))
                #self.degY = 360.0/(4096.0/(0-self.yVal))

            
            if c == 27:
                s.write("0.0,0.0")
                s.close()
                break
            if c == 119:
                s.write("w")
                self.yVal += 1
            if c == 97:
                s.write("a")
                self.xVal += 1
            if c == 115:
                s.write("s")
                self.yVal -= 1
            if c == 100:
                s.write("d")
                self.xVal -= 1
                
            if c == 116:
                s.write("W")
                self.yVal += 10
            if c == 102:
                s.write("A")
                self.xVal += 10
            if c == 103:
                s.write("S")
                self.yVal -= 10
            if c == 104:
                s.write("D")
                self.xVal -= 10

                
            if c == 105:
                s.write("i")
                print self.xVal
                print self.yVal
            if c == 111:
                s.write("o")
                s.write("0.0,0.0")
                self.processing = False
                
            if c == 114:
                s.write("r")
            if c == 122:
                self.xVal = 0
                self.yVal = 0
                
            

if __name__=="__main__":
    t = Target()
    t.run()




